;============================================
; Solar OS sample
; to be used with Solar Assembler
;--------------------------------------------
; This sample shows how to use a 
; FS list API to easy draw a folder view
;============================================


;----------------------------
; import from this DLL
;----------------------------
import_dll ..\sol_os_kernel.dll

;----------------------------
; the following functions
;----------------------------
	import_func	Window_Create
	import_func	Window_Set_Caption
	import_func	Window_Set_Callback
	import_func	Window_Set_Min_Size
	import_func	Window_Get_Rect

	import_func	Mem_Alloc_4k
	import_func	Memory_Release

	import_func	Text_Draw

	import_func	Str_Printf
	import_func	Str_Copy_Nr_Max
	import_func	Str_Copy

	import_func	Fdisk_Drives_Enumerate
	import_func	Fs_Draw_Directory


;----------------------
; define sections
;----------------------
section "code"		class_code
section "data" 		class_data
section	"idata"		class_imports
section "reloc" 	class_relocs


;---------------------
;define entry point
;---------------------
.entry App_Init


;------------------------------
; includes
;------------------------------
include		..\_include\system_32_equ.asm
include		..\_include\system_32_stru.asm


.data
	main_handle		dd	0
	main_caption		db	"Sol FS List Sample",0


	file_name_fmt		db	"file_%u",0

	drv_count		dd	0
	drv_fmt			db	"Drives:%u",0

	list_mem_size		dd	8	; 8*4k = 32k
	list_ptr		dd	0
	list_handle		dd	0

	list_len		dd	0
	list_start		dd	0
	list_pos		dd	0

	btn_handle		dd	0
	btn_caption		db	"Refresh",0

	tmp_buff1		rb	128
.code



App_Init:	
	;----------------------------------
	; create main application window
	;----------------------------------	
	mov	eax,FLAG_WND_ALPHA+FLAG_WND_MINI
	invoke	Window_Create,0,128,128,320,200,eax,WND_TYPE_TOP
	mov	[main_handle],eax

	;-----------------
	; set caption
	;-----------------
	invoke	Window_Set_Caption, eax, main_caption

	;-----------------
	; set callback
	;-----------------
	invoke	Window_Set_Callback,[main_handle],OS_CALL_AFTER,Main_Callback

	;-------------------------------------
	; set minimum resize values
	;-------------------------------------
	invoke	Window_Set_Min_Size,[main_handle],200,108


	;----------------------------------
	; create refresh button
	;----------------------------------	
	mov	eax,FLAG_WND_NO_MOVE+FLAG_WND_NO_RESIZE+FLAG_WND_KEEP_RIGHT
	invoke	Window_Create,[main_handle],100,30,94,22,eax,WND_TYPE_BUTTON
	mov	[btn_handle],eax

	;-----------------
	; set caption
	;-----------------
	invoke	Window_Set_Caption, eax, btn_caption

	;-----------------
	; set callback
	;-----------------
	invoke	Window_Set_Callback,[btn_handle],OS_CALL_AFTER,Btn_Callback



	;-----------------------------------
	; allocate memory for FS list data
	;-----------------------------------
	invoke	Memory_Allocate,[list_mem_size]
	mov	[list_handle],eax
	mov	[list_ptr],esi

	ret



PROC Fs_List_Init stdcall
	USES	edx,ecx,ebx,esi,edi
	ARG	list_items
	LOCAL	list_fill_size

	;---------------------------------------------
	; Brief:
	; - this will init FS list with dummy items
	;
	; a FAT16 or new filesystem driver should
	; fill up this structure as needed 
	;---------------------------------------------
	mov	ecx,[list_items]
	mov	edi,[list_ptr]

	xor	eax,eax
	mov	[list_fill_size],eax

loop_items:
	;---------------------------
	; make it a long filename
	;----------------------------
	mov	[edi + OS_DIR_LIST.dl_lfn],1

	; dummy file size
	mov	[edi + OS_DIR_LIST.dl_size],12345h

	; dummy file type
	mov	[edi + OS_DIR_LIST.dl_type],0

	; dummy file cluster
	mov	[edi + OS_DIR_LIST.dl_cluster],12

	;-----------------------
	; make-up a filename
	;-----------------------
	invoke	Str_Printf,tmp_buff1,file_name_fmt,ecx
	mov	esi,edi
	add	esi,OS_DIR_LIST.dl_name_long
	invoke	Str_Copy,tmp_buff1,esi

	;---------------------
	; update list size
	;---------------------
	mov	eax,[list_fill_size]
	add	eax,OS_DIR_LIST		; in fact size of
	mov	[list_fill_size],eax

	;---------------------
	; next item/file 
	;---------------------
	add	edi,OS_DIR_LIST		; in fact size of
	dec	ecx
	jnz	loop_items

	;-----------------------------------
	; unfortunately the list length 
	; is in bytes not in items
	; this might change in the future
	;-----------------------------------
	mov	eax,[list_fill_size]
	mov	[list_len],eax

	ret
ENDP



PROC Main_Callback STDCALL
	USES	ebx,edx,esi,edi
	ARG	wnd_handle,wnd_action,wnd_param1,wnd_param2

	;------------------------------------
	; switch on message type
	;------------------------------------
	mov	eax,[wnd_action]

	.IF eax == ACT_PAINT
		;---------------------
		; paint handling
		;---------------------
		invoke	Main_On_Paint,[wnd_handle]

	.elseif eax == ACT_KEY
		;-----------------
		; key handling
		;-----------------
		invoke	Main_On_Key,[wnd_handle],[wnd_param1],[wnd_param2]

	.elseif eax == ACT_CLOSE
		;---------------------
		; free used memory
		;----------------------
		invoke	Memory_Release,[list_handle],[list_mem_size]
	
	.ENDIF

	xor	eax,eax
	ret
ENDP



PROC Main_On_Paint stdcall
	USES	edx,ecx,ebx,esi,edi
	ARG 	wnd_handle
	LOCAL	x1,x2,y1,y2

	;-----------------------
	; paint drive header
	;-----------------------
	invoke	Str_Printf,tmp_buff1,drv_fmt,[drv_count]
	invoke	Text_Draw,[wnd_handle],12,32,tmp_buff1,0ff7f1fh

	;------------------------------
	; do not draw with no drives
	;------------------------------
	mov	eax,[drv_count]
	test	eax,eax
	jz	finish

	;-------------------------
	; get window rectangle
	;-------------------------
	invoke	Window_Get_Rect,[wnd_handle]
	mov	[x1],eax
	mov	[x2],ebx
	mov	[y1],ecx
	mov	[y2],edx

	;--------------------------------
	; position the list in window
	; ie add some margins
	;--------------------------------
	add	[x1],8
	add	[y1],64
	sub	[x2],32
	sub	[y2],32


	;---------------------------
	; drav file list
	;---------------------------
	invoke	Fs_Draw_Directory,[wnd_handle],[x1],[y1],[x2],[y2],[list_ptr],[list_len],[list_start],[list_pos]


finish:
	ret
ENDP



PROC Main_On_Key STDCALL
	USES	ebx,ecx,edx,esi,edi
	ARG	wnd_handle,wnd_param1,wnd_param2

	LOCAL	x1,x2,y1,y2
	LOCAL	dx1,dy1,dy3
	LOCAL	max_lines

	mov	eax,[wnd_param1]

	.IF eax == KEY_EX_ARROW_DOWN
		inc	[list_pos]

	.elseif eax == KEY_EX_ARROW_UP
		.if [list_pos] > 0
			dec	[list_pos]
		.endif

	.elseif eax == KEY_EX_HOME
		xor	eax,eax
		mov	[list_pos],eax

	.ENDIF

	ret
ENDP



PROC Btn_Callback STDCALL
	USES	ebx,edx,esi,edi
	ARG	wnd_handle,wnd_action,wnd_param1,wnd_param2

	;------------------------------------
	; switch on message type
	;------------------------------------
	mov	eax,[wnd_action]

	.IF eax == ACT_LEFT_UP
		;---------------------
		; On Mouse Click
		;---------------------

		;-----------------
		; refresh drives
		;-----------------
		invoke	Fdisk_Drives_Enumerate
		mov	[drv_count],eax

		;---------------------------
		; init a dummy file list
		;---------------------------
		invoke	Fs_List_Init,12
	
	.ENDIF

	xor	eax,eax
	ret
ENDP



;******************************
;  END OF PROGRAM TEXT
;******************************
